%macro CPSLMETL(slmpath=,
                environmentDBPath=,
                contractID=,
                contract=,
                shareService=,
                shareUser=,
                sharePassword=,
                action=,
                _rc=);
 %*+----------------------------------------------------------------+
   | Copyright (c) 2003 by SAS Institute Inc., Cary, NC 27512-8000
   | Name:     CPSLMETL.SAS
   | Support:  CES
   | Product:  SAS IT Service Level Management
   | Language: SAS
   | Purpose:  Run ETL-related tasks for a SLM Contract
   |
   | Parameters:
   |
   |   Name             Description
   |
   |  slmpath           path to the SLM software
   |  environmentDBPath path to the SLM environment
   |  contractID        ID of the contract
   |  contract          Name of the contract
   |  shareService      Service name (or port number) for SHARE
   |  shareUser         Userid for authentication to SHARE
   |  sharePassword     Password for authentication to SHARE
   |  action            VALIDATE | SYNC | ETL
   |  _rc               [optional] Name of macro var to contain RC
   |
   | Notes:
   |    The shareService= parameter is optional, and is only used if
   |    a SAS/SHARE server is used to access the environment database.
   |    The shareUser= and sharePassword= parameters are also optional
   |    and are specified for purposes of authentication to the SHARE
   |    server.
   |
   | end
   +----------------------------------------------------------------+;

   options compress=yes obs=max firstobs=1 nofmterr s=0 s2=0 user=WORK
           validvarname = UPCASE;

   %if %substr(&sysver,1,1) eq 9 %then %do;
      options noquotelenmax;
   %end;

   %******************************* ;
   %* Macro variable declarations * ;
   %******************************* ;
   %global _SLMrc;        %* ITSLM global return code variable ;
   %global SLMDEBUG;      %* ITSLM global debug flag ;
   %global _CPSLMETLrc;   %* return code variable for this macro ;
   %global cppmrc;        %* ITMS Core global return code variable;
   %global cproot;        %* ITMS Core software root ;
   %global cpsow;         %* ITMS Core needs this to be global ;

   %local  accessString;  %* System-specific readonly libname option;

   %local  _action;           %* For upcase comparison ;
   %local  _actionParm;       %* For SCL invocation ;
   %local  _slmpathParm;      %* For SCL invocation ;
   %local  _contractParm;     %* For SCL invocation ;
   %local  _envPathParm;      %* For SCL invocation ;
   %local  _shareServiceParm; %* For SCL invocation ;
   %local  _shareUserParm;    %* For SCL invocation ;
   %local  _sharePasswordParm;%* For SCL invocation ;
   %local  _debugParm;        %* For SCL invocation ;
   %local  _windowPrefix;     %* For SCL invocation ;

   %local  _parmRC;           %* _rc is being overwritten ;

   %local  CPSLMETL_notes;     %* save and reset ;
   %local  CPSLMETL_source;    %* save and reset ;
   %local  CPSLMETL_source2;   %* save and reset ;
   %local  CPSLMETL_mprint;    %* save and reset ;
   %local  CPSLMETL_mlogic;    %* save and reset ;
   %local  CPSLMETL_macrogen;  %* save and reset ;
   %local  CPSLMETL_symbolgen; %* save and reset ;

   %***************************************************************;
   %* Save some option settings to keep the noise down in the log *;
   %***************************************************************;
   %let CPSLMETL_notes   = %SYSFUNC(getoption(notes));
   %let CPSLMETL_source  = %SYSFUNC(getoption(source));
   %let CPSLMETL_source2 = %SYSFUNC(getoption(source2));
   %let CPSLMETL_mprint  = %SYSFUNC(getoption(mprint));
   %let CPSLMETL_mlogic  = %SYSFUNC(getoption(mlogic));

    %if &SLMDEBUG eq 1 %then %do;
       options notes source source2 mprint mlogic macrogen symbolgen;
   %end;
   %else %do;
       options nonotes  nosource   nosource2 nomprint
               nomlogic nomacrogen nosymbolgen;
   %end;

   %let _CPSLMETLrc = 99;  %* Assume a bad RC;
   %if %length(&_rc) eq 0 %then %do;
       %local RC;
       %let _rc = RC;
   %end;
   %else %do;
        %let _parmRC = &_rc;
        %let &_rc = &_CPSLMETLrc;
   %end;

   %********************************* ;
   %* Determine host-specific stuff * ;
   %********************************* ;
   %if %cphsttyp eq MVS %then %do;
       %let accessString = %str(disp=shr);
   %end;
   %else %do;
       %let accessString = %str(access=readonly);
   %end;

   %****************** ;
   %* Internal macro * ;
   %****************** ;
   %macro _releaseLib(name);
       %if %sysfunc(libref(&name)) eq 0 %then %do;
           libname &name clear;
       %end;
   %mend _releaseLib;

   %put NOTE:(&SYSMACRONAME)  SLM2000 SAS IT Service Level Management
initialization in progress.;

   %********************************** ;
   %* Validate the ACTION= parameter * ;
   %********************************** ;
   %let _action = %UPCASE(&action);
   %if ((&_action ne SYNC) and
        (&_action ne VALIDATE) and
        (&_action ne ETL)) %then %do;
       %let _SLMrc= 8;
       %put ERROR:(&SYSMACRONAME)  SLM2001 Invalid action= parameter
value specified: &action;
       %goto EXIT;
   %end;
   %else %do;
       %let _actionParm = ACTION=&_action;
   %end;

   %************************************** ;
   %* Massage the SLM environment parm.  * ;
   %* Be sure the pathname is not blank, * ;
   %* unless SHARE is being used.        * ;
   %************************************** ;
    %if (%length(&environmentDBPath) le 0) and
        (%length(&shareService) le 0) %then %do;
       %let _SLMrc = 8;
       %put ERROR:(&SYSMACRONAME)  SLM2002 Parameter EnvironmentDBPath
not specified or zero length.;
       %goto EXIT;
   %end;
   %let _envPathParm = ENVIRONMENT=%str(%'&environmentDBPath%');

   %****************************************************** ;
   %* Validate the CONTRACTID= and CONTRACT= parameters. * ;
   %* We must have one of these parameters.  The SCL     * ;
   %* code will handle verifying that whatever is given  * ;
   %* is valid.  Here we just want to make sure we have  * ;
   %* exactly one of these parameters specified.         * ;
   %****************************************************** ;
   %if ((%length(&contractID) eq 0) and
        (%length(&contract) eq 0)) %then %do;
       %let _SLMrc= 8;
       %put ERROR:(&SYSMACRONAME)  SLM2003 Neither contractid= or
contract= parameters were specified.;
       %goto EXIT;
   %end;

   %if ((%length(&contractID) gt 0) and
        (%length(&contract) gt 0)) %then %do;
       %let _SLMrc= 8;
       %put ERROR:(&SYSMACRONAME)  SLM2004 Both contractid= or
contract= parameters were specified.;
       %goto EXIT;
   %end;

   %if (%length(&contractID) gt 0) %then %do;
       %let _contractParm = CONTRACTID=&contractID;
   %end;

   %if (%length(&contract) gt 0) %then %do;
       %let _contractParm = CONTRACT='&contract';
   %end;

   %****************************************;
   %* Add SHARE-related parameters, if any *;
   %****************************************;
   %if (%length(&shareService) gt 0) %then %do;
      %let _shareServiceParm = SHARESERVICE=&shareService;
      %if (%length(&shareUser) gt 0) %then
         %let _shareUserParm = SHAREUSER=%str(%'&shareUser%');
      %if (%length(&sharePassword) gt 0) %then
         %let _sharePasswordParm = SHAREPASSWORD=%str(%'&sharePassword%');
   %end;

   %******************************************* ;
   %* Allocate SLMLIB to access the software. * ;
   %******************************************* ;
   %cpslmlib(slmpath=&slmpath,_rc=_SLMrc);
   %if (&_SLMrc ne 0) %then %do ;
       %put ERROR:(&SYSMACRONAME)  SLM2006 Allocation failed for
&slmpath;
       %goto EXIT;
   %end;

   %************************************************************* ;
   %* Set the AF DEBUG= parm based upon the setting of SLMDEBUG * ;
   %************************************************************* ;
   %if &slmdebug eq 1 and &SYSENV eq FORE %then %do;
      %let _debugParm = YES;
   %end;
   %else %do;
      %let _debugParm = NO;
   %end;

   %********************************* ;
   %* Turn off confounding windows  * ;
   %********************************* ;
   %if %cphsttyp eq WIN %then %do;
       %local _dmsexp;
       proc sql noprint;
          select setting into :_dmsexp from dictionary.options
             where optname='DMSEXP';
       %if &_dmsexp eq DMSEXP %then %do;
          %let _windowPrefix =
            %str( ODSRESULTS; CAN; NEXT 'EXPLORER'; CAN; );
       %end;
   %end;

   %**************************************************** ;
   %* call the SCL code (return code is set in _SLMRC) * ;
   %**************************************************** ;
   dm "&_windowPrefix afa c=slmlib.slmenv.slmbatch.scl
      title='IT Service Level Management'
      aws=YES &_actionParm &_contractParm &_envPathParm &_slmpathParm
      &_shareServiceParm &_shareUserParm &_sharePasswordParm
      debug=&_debugParm" afa;

%EXIT:
   %_releaseLib(SLM)
   %_releaseLib(SLMLIB)
   %************************************************************** ;
   %* clear any librefs related to the contract PDB or ITMS Core * ;
   %************************************************************** ;
   %_releaseLib(PGMLIB)
   %_releaseLib(SITELIB)
   %_releaseLib(DETAIL)
   %_releaseLib(DAY)
   %_releaseLib(WEEK)
   %_releaseLib(MONTH)
   %_releaseLib(YEAR)
   %_releaseLib(PDBWORK)
   %_releaseLib(ADMIN)
   %_releaseLib(COLLECT)
   %_releaseLib(DICTLIB)

   %******************************************** ;
   %* Populate user return code macro variable * ;
   %******************************************** ;
   %if %eval(&_SLMrc gt .) %then %do;
       %let _CPSLMETLrc = &_SLMrc;
   %end;
   %else %do;
       %let _SLMrc = &_CPSLMETLrc;
   %end;
   %if %length(&_parmRC) gt 0 %then %do;
       %let &_parmRC = &_CPSLMETLrc;
   %end;
   %put NOTE:(&SYSMACRONAME)  SLM2008 SAS IT Service Level Management
ended with return code &_CPSLMETLrc;
   options &CPSLMETL_notes  &CPSLMETL_source  &CPSLMETL_source2
           &CPSLMETL_mprint &CPSLMETL_mlogic  &CPSLMETL_macrogen
           &CPSLMETL_symbolgen ;
%mend CPSLMETL;
